@@ -1,7 +1,7 @@
 /*
 
    xlog - GTK+ logging program for amateur radio operators
-   Copyright (C) 2012 - 2019 Andy Stewart <kb1oiq@arrl.net>
+   Copyright (C) 2012 - 2020 Andy Stewart <kb1oiq@arrl.net>
    Copyright (C) 2001 - 2008 Joop Stakenborg <pg4i@amsat.org>
 
    This file is part of xlog.
@@ -67,17 +67,17 @@ static gint adif_qso_foreach
   memset (q, 0, sizeof (q));
   buffer = g_strdup ("");
   while (!feof (fp))
-    {
-      if (!fgets (line, 1023, fp)) break;
+  {
+    if (!fgets (line, 1023, fp)) break;
 
-      if (line[0] != '\n') /* ignore empty lines */
-	buffer = g_strconcat (buffer, line, NULL);
-      else continue;
+	if (line[0] != '\n') /* ignore empty lines */
+	  buffer = g_strconcat (buffer, line, NULL);
+	else continue;
 
-      if (g_strrstr (line, "<EOH>") || g_strrstr (line, "<eoh>")) /* read past header */
-	buffer = g_strdup ("");
+	if (g_strrstr (line, "<EOH>") || g_strrstr (line, "<eoh>")) /* read past header */
+	  buffer = g_strdup ("");
 
-      if (g_strrstr (line, "<EOR>") || g_strrstr (line, "<eor>")) /* read past one record */
+	if (g_strrstr (line, "<EOR>") || g_strrstr (line, "<eor>")) /* read past one record */
 	{
 	  /* let's do this simple, check if we have multiple '<' */
 	  buffer = my_strreplace (buffer, "\n", "");
@@ -86,202 +86,202 @@ static gint adif_qso_foreach
 	    if (!adifline || adifline[items] == NULL) break;
 
 	  if (items > 2) /* we have a valid adif line*/
-	    {
-	      for (i = 0; i < items; i++)
-		if (g_strrstr (adifline[i], ">")
-		    && g_strrstr (adifline[i], ":"))
-		  /* valid adif item */
-		  {
-		    adifitem = g_strsplit (adifline[i], ">", 2);
-		    /* adifitem [0] contains 'id' and string length */
-		    adifid = g_strsplit (adifitem[0], ":", -1);
-		    /* that's all we need, now fill in the fields */
-		    if (!strcasecmp (adifid[0], "CALL"))
+	  {
+		for (i = 0; i < items; i++)
+		  if (g_strrstr (adifline[i], ">")
+			  && g_strrstr (adifline[i], ":"))
+			/* valid adif item */
+			{
+			  adifitem = g_strsplit (adifline[i], ">", 2);
+			  /* adifitem [0] contains 'id' and string length */
+			  adifid = g_strsplit (adifitem[0], ":", -1);
+			  /* that's all we need, now fill in the fields */
+			  if (!strcasecmp (adifid[0], "CALL"))
 		      {
-			adif_field = CALL;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
+				adif_field = CALL;
+				q[adif_field] =
+				  g_strndup (adifitem[1], atoi(adifid[1]));
 		      }
-		    else if (!strcasecmp (adifid[0], "QSO_DATE"))
+			  else if (!strcasecmp (adifid[0], "QSO_DATE"))
 		      {
-			sscanf(adifitem[1], "%4d%2d%2d", &timestruct.tm_year,
-			       &timestruct.tm_mon,	&timestruct.tm_mday);
-			timestruct.tm_year -= 1900;
-			timestruct.tm_mon--;
-			strftime (sdate, 16, "%d %b %Y", &timestruct);
-			adif_field = DATE;
-			q[adif_field] = g_strdup (sdate);
-		      }
-		    else if (!strcasecmp (adifid[0], "TIME_ON"))
-		      {
-			adif_field = GMT;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "TIME_OFF"))
-		      {
-			adif_field = GMTEND;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "BAND"))
-		      {
-			/* prefer FREQ over BAND */
-			if (!q[BAND])
-			  {
-			    adif_field = BAND;
-			    guint enumband = meters2enum (adifitem[1]);
-			    q[adif_field] = band_enum2char (enumband);
-			  }
-		      }
-		    else if (!strcasecmp (adifid[0], "FREQ"))
-		      {
-			/* prefer FREQ over BAND */
-			if (q[BAND])
-			  {
-			    g_free(q[BAND]);
-			    q[BAND] = NULL;
-			  }
-			adif_field = BAND;
-			if ((p = g_strrstr(adifitem[1], ",")))
-			  *p = '.';
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
+				sscanf(adifitem[1], "%4d%2d%2d", &timestruct.tm_year,
+					   &timestruct.tm_mon,	&timestruct.tm_mday);
+				timestruct.tm_year -= 1900;
+				timestruct.tm_mon--;
+				strftime (sdate, 16, "%d %b %Y", &timestruct);
+				adif_field = DATE;
+				q[adif_field] = g_strdup (sdate);
 		      }
+			  else if (!strcasecmp (adifid[0], "TIME_ON"))
+				{
+				  adif_field = GMT;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "TIME_OFF"))
+				{
+				  adif_field = GMTEND;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "BAND"))
+				{
+				  /* prefer FREQ over BAND */
+				  if (!q[BAND])
+					{
+					  adif_field = BAND;
+					  guint enumband = meters2enum (adifitem[1]);
+					  q[adif_field] = band_enum2char (enumband);
+					}
+				}
+			  else if (!strcasecmp (adifid[0], "FREQ"))
+				{
+				  /* prefer FREQ over BAND */
+				  if (q[BAND])
+					{
+					  g_free(q[BAND]);
+					  q[BAND] = NULL;
+					}
+				  adif_field = BAND;
+				  if ((p = g_strrstr(adifitem[1], ",")))
+					*p = '.';
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
 
-		    /* MODE and SUBMODE might both be specified as of ADIF 3.0.4.
-		       Assume(!) that they are consistent in the imported ADIF3 file.
-		       Give precedence to the SUBMODE if both occur.
-		    */
+			  /* MODE and SUBMODE might both be specified as of ADIF 3.0.4.
+				 Assume(!) that they are consistent in the imported ADIF3 file.
+				 Give precedence to the SUBMODE if both occur.
+			  */
 
-		    else if (!strcasecmp (adifid[0], "MODE"))
-		      {
-			adif_field = MODE;
-			if (!q[adif_field]) {
-			  q[adif_field] = g_strndup (adifitem[1], atoi(adifid[1]));
+			  else if (!strcasecmp (adifid[0], "MODE"))
+				{
+				  adif_field = MODE;
+				  if (!q[adif_field]) {
+					q[adif_field] = g_strndup (adifitem[1], atoi(adifid[1]));
+				  }
+				}
+			  else if (!strcasecmp (adifid[0], "SUBMODE"))
+				{
+				  adif_field = MODE;
+				  if (q[adif_field]) {
+					g_free(q[adif_field]);
+					q[adif_field] = NULL;
+				  }
+				  q[adif_field] = g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "TX_PWR"))
+				{
+				  adif_field = POWER;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "RST_SENT"))
+				{
+				  adif_field = RST;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "RST_RCVD"))
+				{
+				  adif_field = MYRST;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "QSL_SENT"))
+				{
+				  adif_field = QSLOUT;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "QSL_RCVD"))
+				{
+				  adif_field = QSLIN;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "QSL_VIA"))
+				{
+				  if (!strcasecmp (preferences.freefield1, "QSL_VIA"))
+					{
+					  adif_field = U1;
+					  q[adif_field] =
+						g_strndup (adifitem[1], atoi(adifid[1]));
+					}
+				  else if (!strcasecmp (preferences.freefield2, "QSL_VIA"))
+					{
+					  adif_field = U2;
+					  q[adif_field] =
+						g_strndup (adifitem[1], atoi(adifid[1]));
+					}
+				}
+			  else if (!strcasecmp (adifid[0], "NAME"))
+				{
+				  adif_field = NAME;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "QTH"))
+				{
+				  adif_field = QTH;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "GRIDSQUARE"))
+				{
+				  adif_field = LOCATOR;
+				  q[adif_field] =
+					g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "COMMENT"))
+				{
+				  adif_field = REMARKS;
+				  /* append comment to remarks */
+				  if (q[REMARKS])
+					{
+					  tmp = g_strndup (adifitem[1], atoi(adifid[1]));
+					  q[adif_field] =
+						g_strconcat (q[adif_field], ", ", tmp, NULL);
+					  g_free (tmp);
+					}
+				  else
+					q[adif_field] =
+					  g_strndup (adifitem[1], atoi(adifid[1]));
+				}
+			  else if (!strcasecmp (adifid[0], "NOTES"))
+				{
+				  adif_field = REMARKS;
+				  /* append notes to remarks */
+				  if (q[REMARKS])
+					{
+					  tmp = g_strndup (adifitem[1], atoi(adifid[1]));
+					  q[adif_field] =
+						g_strconcat (q[adif_field], ", ", tmp, NULL);
+					  g_free (tmp);
+					}
+				  else
+					q[adif_field] =
+					  g_strndup (adifitem[1], atoi(adifid[1]));
+
+				}
+			  g_strfreev (adifid);
+			  g_strfreev (adifitem);
 			}
-		      }
-		    else if (!strcasecmp (adifid[0], "SUBMODE"))
-		      {
-			adif_field = MODE;
-			if (q[adif_field]) {
-			  g_free(q[adif_field]);
-			  q[adif_field] = NULL;
-			}
-			q[adif_field] = g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "TX_PWR"))
-		      {
-			adif_field = POWER;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "RST_SENT"))
-		      {
-			adif_field = RST;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "RST_RCVD"))
-		      {
-			adif_field = MYRST;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "QSL_SENT"))
-		      {
-			adif_field = QSLOUT;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "QSL_RCVD"))
-		      {
-			adif_field = QSLIN;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "QSL_VIA"))
-		      {
-			if (!strcasecmp (preferences.freefield1, "QSL_VIA"))
-			  {
-			    adif_field = U1;
-			    q[adif_field] =
-			      g_strndup (adifitem[1], atoi(adifid[1]));
-			  }
-			else if (!strcasecmp (preferences.freefield2, "QSL_VIA"))
-			  {
-			    adif_field = U2;
-			    q[adif_field] =
-			      g_strndup (adifitem[1], atoi(adifid[1]));
-			  }
-		      }
-		    else if (!strcasecmp (adifid[0], "NAME"))
-		      {
-			adif_field = NAME;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "QTH"))
-		      {
-			adif_field = QTH;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "GRIDSQUARE"))
-		      {
-			adif_field = LOCATOR;
-			q[adif_field] =
-			  g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "COMMENT"))
-		      {
-			adif_field = REMARKS;
-			/* append comment to remarks */
-			if (q[REMARKS])
-			  {
-			    tmp = g_strndup (adifitem[1], atoi(adifid[1]));
-			    q[adif_field] =
-			      g_strconcat (q[adif_field], ", ", tmp, NULL);
-			    g_free (tmp);
-			  }
-			else
-			  q[adif_field] =
-			    g_strndup (adifitem[1], atoi(adifid[1]));
-		      }
-		    else if (!strcasecmp (adifid[0], "NOTES"))
-		      {
-			adif_field = REMARKS;
-			/* append notes to remarks */
-			if (q[REMARKS])
-			  {
-			    tmp = g_strndup (adifitem[1], atoi(adifid[1]));
-			    q[adif_field] =
-			      g_strconcat (q[adif_field], ", ", tmp, NULL);
-			    g_free (tmp);
-			  }
-			else
-			  q[adif_field] =
-			    g_strndup (adifitem[1], atoi(adifid[1]));
-
-		      }
-		    g_strfreev (adifid);
-		    g_strfreev (adifitem);
-		  }
-	    }
+	  }
 	  g_strfreev (adifline);
 	  buffer = g_strdup ("");
 	}
-      if (adif_field != -1)
-	{
-	  /* fill in empty fields */
-	  for (i = 0; i < adif_field_nr; i++)
-	    if ( !q[adif_fields[i]] ) q[adif_fields[i]] = g_strdup ("");
-	  ret = (*adif_fn)(adif_handle, q, adif_arg);
-	  adif_field = -1;
-	  memset(q, 0, sizeof(q));
-	  if (ret) return ret;
-	}
-    }
+	if (adif_field != -1)
+	  {
+		/* fill in empty fields */
+		for (i = 0; i < adif_field_nr; i++)
+		  if ( !q[adif_fields[i]] ) q[adif_fields[i]] = g_strdup ("");
+		ret = (*adif_fn)(adif_handle, q, adif_arg);
+		adif_field = -1;
+		memset(q, 0, sizeof(q));
+		if (ret) return ret;
+	  }
+  }
   g_free (buffer);
   return 0;
 }
@@ -327,7 +327,7 @@ gint adif_create(LOGDB *handle)
 
 	/* write header */
 	fprintf(fp, "ADIF Export from " PACKAGE " Version " VERSION "\n"
-				"Copyright (C) 2012-2017 Andy Stewart <kb1oiq@arrl.net>\n"
+				"Copyright (C) 2012-2020 Andy Stewart <kb1oiq@arrl.net>\n"
 				"Copyright (C) 2001-2010 Joop Stakenborg <pg4i@amsat.org>\n"
 				"Internet:  http://savannah.nongnu.org/projects/Xlog\n\n"
 				"Date of export: %s\n",
@@ -341,8 +341,7 @@ gint adif_create(LOGDB *handle)
 		fprintf(fp, "Locator: %s\n", preferences.locator);
 	}
 
-	fprintf(fp, 		"\n<ADIF_VER:5>3.1.0\n"
-				"<EOH>\n");
+	fprintf(fp, "\n<ADIF_VER:5>3.1.1\n" "<EOH>\n");
 	return 0;
 }
 
@@ -379,8 +378,17 @@ static const gchar *xlog2adif_name(gint fld)
 		case NAME: return "NAME";
 		case QTH: return "QTH";
 		case LOCATOR: return "GRIDSQUARE";
-		case U1: return preferences.freefield1;
-		case U2: return preferences.freefield2;
+		case U1:
+		  if (!strcmp (preferences.freefield1, "Distance"))
+			return "DISTANCE";
+		  else
+			return preferences.freefield1;
+		case U2:
+		  if (!strcmp (preferences.freefield2, "Azimuth"))
+			return "ANT_AZ";
+		  else
+			return preferences.freefield2;
+			  
 		default: return "UNKNOWN";
 	}
 }
@@ -402,7 +410,7 @@ gint adif_qso_append(LOGDB *handle, const qso_t *q)
 	{
 		gint qfield_len, number;
 		gint fld = handle->column_fields[i];
-		gchar *qfield, *serial = NULL, *endptr = NULL;
+		gchar *qfield = NULL, *serial = NULL, *endptr = NULL, *space_ptr = NULL, *tmpfield;
 
 		if (!q[fld])
 			continue;
@@ -495,6 +503,22 @@ gint adif_qso_append(LOGDB *handle, const qso_t *q)
 		  qfield_len = strlen(qfield);
 		}
 
+		else if (((fld == U1) && (!strcmp (xlog2adif_name(fld), "DISTANCE"))) ||
+				 ((fld == U2) && (!strcmp (xlog2adif_name(fld), "ANT_AZ"))))
+        {
+		  tmpfield = q[fld];
+		  space_ptr = q[fld];
+		  space_ptr = strpbrk (space_ptr, " ");
+		  qfield_len = space_ptr - tmpfield;
+
+		  if (qfield_len == 0) {
+			qfield = q[fld];
+		  } else {
+			strncpy (qfield, tmpfield, qfield_len);
+			qfield[qfield_len] = '\0';
+		  }
+		}
+
 		else
 		{
 			qfield = q[fld];
@@ -526,7 +550,7 @@ gint adif_qso_append(LOGDB *handle, const qso_t *q)
 		}
 
 	}
-	fprintf(fp, "\n<EOR>\n");
+	fprintf(fp, "<EOR>\n");
 
 	return 0;
 }
@@ -726,6 +750,7 @@ gint adif3_mode2enum (gchar * str)
  	if 
  	( 
  		!g_ascii_strcasecmp (str, "FSQCALL") ||
+ 		!g_ascii_strcasecmp (str, "FST4")    ||
  		!g_ascii_strcasecmp (str, "FT4")     ||
  		!g_ascii_strcasecmp (str, "JS8")     ||
  		!g_ascii_strcasecmp (str, "MFSK")    ||
